@@ -1,5 +1,5 @@
-use super::MatchFilterFunc;
-use super::{Desc, ExecutionNode, NodeSearchDesc};
+use super::MatchValueFilterFunc;
+use super::{ExecutionNode, ExecutionNodeDesc, NodeSearchDesc};
 use crate::annis::db::exec::tokensearch;
 use crate::annis::db::exec::tokensearch::AnyTokenSearch;
 use crate::annis::db::{aql::model::AnnotationComponentType, AnnotationStorage};
@@ -11,6 +11,7 @@ use crate::{
     annis::{db::aql::model::TOKEN_KEY, util},
     graph::Match,
 };
+use graphannis_core::graph::{ANNIS_NS, NODE_NAME};
 use graphannis_core::{
     annostorage::{MatchGroup, ValueSearch},
     graph::{storage::GraphStorage, NODE_TYPE_KEY},
@@ -27,10 +28,15 @@ pub struct NodeSearch<'a> {
     /// The actual search implementation
     it: Box<dyn Iterator<Item = MatchGroup> + 'a>,
 
-    desc: Option<Desc>,
+    desc: Option<ExecutionNodeDesc>,
     node_search_desc: Arc<NodeSearchDesc>,
     is_sorted: bool,
 }
+struct NodeDescArg {
+    query_fragment: String,
+    node_nr: usize,
+}
+
 #[derive(Clone, Debug, PartialOrd, Ord, Hash, PartialEq, Eq)]
 pub enum NodeSearchSpec {
     ExactValue {
@@ -76,20 +82,6 @@ pub enum NodeSearchSpec {
 }
 
 impl NodeSearchSpec {
-    pub fn new_exact(
-        ns: Option<&str>,
-        name: &str,
-        val: Option<&str>,
-        is_meta: bool,
-    ) -> NodeSearchSpec {
-        NodeSearchSpec::ExactValue {
-            ns: ns.map(String::from),
-            name: String::from(name),
-            val: val.map(String::from),
-            is_meta,
-        }
-    }
-
     pub fn necessary_components(
         &self,
         db: &AnnotationGraph,
@@ -99,6 +91,187 @@ impl NodeSearchSpec {
         }
         HashSet::default()
     }
+
+    /// Get the annotatiom qualified name needed to execute a search with this specification.
+    pub fn get_anno_qname(&self) -> (Option<String>, Option<String>) {
+        match self {
+            NodeSearchSpec::ExactValue { ns, name, .. }
+            | NodeSearchSpec::NotExactValue { ns, name, .. }
+            | NodeSearchSpec::RegexValue { ns, name, .. }
+            | NodeSearchSpec::NotRegexValue { ns, name, .. } => {
+                (ns.to_owned(), Some(name.to_owned()))
+            }
+            NodeSearchSpec::ExactTokenValue { .. }
+            | NodeSearchSpec::NotExactTokenValue { .. }
+            | NodeSearchSpec::RegexTokenValue { .. }
+            | NodeSearchSpec::NotRegexTokenValue { .. }
+            | NodeSearchSpec::AnyToken { .. } => (
+                Some(TOKEN_KEY.ns.clone().into()),
+                Some(TOKEN_KEY.name.clone().into()),
+            ),
+            NodeSearchSpec::AnyNode => (
+                Some(NODE_TYPE_KEY.ns.clone().into()),
+                Some(NODE_TYPE_KEY.name.clone().into()),
+            ),
+        }
+    }
+
+    /// Creates a vector of value filter functions for this node annotation search.
+    pub fn get_value_filter(
+        &self,
+        g: &AnnotationGraph,
+        location_in_query: Option<LineColumnRange>,
+    ) -> Result<Vec<MatchValueFilterFunc>> {
+        let mut filters: Vec<MatchValueFilterFunc> = Vec::new();
+        match self {
+            NodeSearchSpec::ExactValue { val, .. } => {
+                if let Some(val) = val {
+                    let val: String = val.to_owned();
+                    filters.push(Box::new(move |m, node_annos| {
+                        if let Some(anno_val) = node_annos.get_value_for_item(&m.node, &m.anno_key)
+                        {
+                            anno_val == val.as_str()
+                        } else {
+                            false
+                        }
+                    }));
+                }
+            }
+            NodeSearchSpec::NotExactValue { val, .. } => {
+                let val: String = val.to_owned();
+                filters.push(Box::new(move |m, node_annos| {
+                    if let Some(anno_val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
+                        anno_val != val.as_str()
+                    } else {
+                        false
+                    }
+                }));
+            }
+            NodeSearchSpec::RegexValue { val, .. } => {
+                let full_match_pattern = graphannis_core::util::regex_full_match(val);
+                let re = regex::Regex::new(&full_match_pattern);
+                match re {
+                    Ok(re) => {
+                        filters.push(Box::new(move |m, node_annos| {
+                            if let Some(val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
+                                re.is_match(&val)
+                            } else {
+                                false
+                            }
+                        }));
+                    }
+                    Err(e) => {
+                        return Err(GraphAnnisError::AQLSemanticError(AQLError {
+                            desc: format!("/{}/ -> {}", val, e),
+                            location: location_in_query,
+                        }));
+                    }
+                }
+            }
+            NodeSearchSpec::NotRegexValue { val, .. } => {
+                let full_match_pattern = graphannis_core::util::regex_full_match(val);
+                let re = regex::Regex::new(&full_match_pattern);
+                match re {
+                    Ok(re) => {
+                        filters.push(Box::new(move |m, node_annos| {
+                            if let Some(val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
+                                !re.is_match(&val)
+                            } else {
+                                false
+                            }
+                        }));
+                    }
+                    Err(e) => {
+                        return Err(GraphAnnisError::AQLSemanticError(AQLError {
+                            desc: format!("/{}/ -> {}", val, e),
+                            location: location_in_query,
+                        }));
+                    }
+                }
+            }
+            NodeSearchSpec::ExactTokenValue { val, leafs_only } => {
+                let val = val.clone();
+                filters.push(Box::new(move |m, node_annos| {
+                    if let Some(anno_val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
+                        anno_val == val.as_str()
+                    } else {
+                        false
+                    }
+                }));
+                if *leafs_only {
+                    filters.push(create_token_leaf_filter(g));
+                }
+            }
+            NodeSearchSpec::NotExactTokenValue { val } => {
+                let val = val.clone();
+                filters.push(Box::new(move |m, node_annos| {
+                    if let Some(anno_val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
+                        anno_val != val.as_str()
+                    } else {
+                        false
+                    }
+                }));
+            }
+            NodeSearchSpec::RegexTokenValue { val, leafs_only } => {
+                let full_match_pattern = graphannis_core::util::regex_full_match(val);
+                let re = regex::Regex::new(&full_match_pattern);
+                match re {
+                    Ok(re) => filters.push(Box::new(move |m, node_annos| {
+                        if let Some(val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
+                            re.is_match(&val)
+                        } else {
+                            false
+                        }
+                    })),
+                    Err(e) => {
+                        return Err(GraphAnnisError::AQLSemanticError(AQLError {
+                            desc: format!("/{}/ -> {}", val, e),
+                            location: location_in_query,
+                        }));
+                    }
+                };
+                if *leafs_only {
+                    filters.push(create_token_leaf_filter(g));
+                }
+            }
+            NodeSearchSpec::NotRegexTokenValue { val } => {
+                let full_match_pattern = graphannis_core::util::regex_full_match(val);
+                let re = regex::Regex::new(&full_match_pattern);
+                match re {
+                    Ok(re) => filters.push(Box::new(move |m, node_annos| {
+                        if let Some(val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
+                            !re.is_match(&val)
+                        } else {
+                            false
+                        }
+                    })),
+                    Err(e) => {
+                        return Err(GraphAnnisError::AQLSemanticError(AQLError {
+                            desc: format!("/{}/ -> {}", val, e),
+                            location: location_in_query,
+                        }));
+                    }
+                };
+            }
+            NodeSearchSpec::AnyToken => {
+                filters.push(create_token_leaf_filter(g));
+            }
+            NodeSearchSpec::AnyNode => {
+                let filter_func: Box<
+                    dyn Fn(&Match, &dyn AnnotationStorage<NodeID>) -> bool + Send + Sync,
+                > = Box::new(move |m, node_annos| {
+                    if let Some(val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
+                        val == "node"
+                    } else {
+                        false
+                    }
+                });
+                filters.push(filter_func)
+            }
+        }
+
+        Ok(filters)
+    }
 }
 
 impl fmt::Display for NodeSearchSpec {
@@ -190,6 +363,32 @@ impl fmt::Display for NodeSearchSpec {
     }
 }
 
+fn create_token_leaf_filter(g: &AnnotationGraph) -> MatchValueFilterFunc {
+    let cov_gs: Vec<Arc<dyn GraphStorage>> = g
+        .get_all_components(Some(AnnotationComponentType::Coverage), None)
+        .into_iter()
+        .filter_map(|c| g.get_graphstorage(&c))
+        .filter(|gs| {
+            if let Some(stats) = gs.get_statistics() {
+                stats.nodes > 0
+            } else {
+                true
+            }
+        })
+        .collect();
+
+    let filter_func: Box<dyn Fn(&Match, &dyn AnnotationStorage<NodeID>) -> bool + Send + Sync> =
+        Box::new(move |m, _| {
+            for cov in cov_gs.iter() {
+                if cov.get_outgoing_edges(m.node).next().is_some() {
+                    return false;
+                }
+            }
+            true
+        });
+    filter_func
+}
+
 impl<'a> NodeSearch<'a> {
     pub fn from_spec(
         spec: NodeSearchSpec,
@@ -199,6 +398,8 @@ impl<'a> NodeSearch<'a> {
     ) -> Result<NodeSearch<'a>> {
         let query_fragment = format!("{}", spec);
 
+        let filters = spec.get_value_filter(db, location_in_query)?;
+
         match spec {
             NodeSearchSpec::ExactValue {
                 ns,
@@ -209,6 +410,7 @@ impl<'a> NodeSearch<'a> {
                 db,
                 (ns, name),
                 val.into(),
+                filters,
                 is_meta,
                 &query_fragment,
                 node_nr,
@@ -222,6 +424,7 @@ impl<'a> NodeSearch<'a> {
                 db,
                 (ns, name),
                 ValueSearch::NotSome(val),
+                filters,
                 is_meta,
                 &query_fragment,
                 node_nr,
@@ -240,18 +443,19 @@ impl<'a> NodeSearch<'a> {
                         (ns, name),
                         &val,
                         false,
+                        filters,
                         is_meta,
-                        super::NodeDescArg {
+                        NodeDescArg {
                             query_fragment,
                             node_nr,
                         },
-                        location_in_query,
                     )
                 } else {
                     NodeSearch::new_annosearch_exact(
                         db,
                         (ns, name),
                         ValueSearch::Some(val),
+                        filters,
                         is_meta,
                         &query_fragment,
                         node_nr,
@@ -272,18 +476,19 @@ impl<'a> NodeSearch<'a> {
                         (ns, name),
                         &val,
                         true,
+                        filters,
                         is_meta,
-                        super::NodeDescArg {
+                        NodeDescArg {
                             query_fragment,
                             node_nr,
                         },
-                        location_in_query,
                     )
                 } else {
                     NodeSearch::new_annosearch_exact(
                         db,
                         (ns, name),
                         ValueSearch::NotSome(val),
+                        filters,
                         is_meta,
                         &query_fragment,
                         node_nr,
@@ -293,38 +498,38 @@ impl<'a> NodeSearch<'a> {
             NodeSearchSpec::ExactTokenValue { val, leafs_only } => NodeSearch::new_tokensearch(
                 db,
                 ValueSearch::Some(val),
+                filters,
                 leafs_only,
                 false,
                 &query_fragment,
                 node_nr,
-                location_in_query,
             ),
             NodeSearchSpec::NotExactTokenValue { val } => NodeSearch::new_tokensearch(
                 db,
                 ValueSearch::NotSome(val),
+                filters,
                 true,
                 false,
                 &query_fragment,
                 node_nr,
-                location_in_query,
             ),
             NodeSearchSpec::RegexTokenValue { val, leafs_only } => NodeSearch::new_tokensearch(
                 db,
                 ValueSearch::Some(val),
+                filters,
                 leafs_only,
                 true,
                 &query_fragment,
                 node_nr,
-                location_in_query,
             ),
             NodeSearchSpec::NotRegexTokenValue { val } => NodeSearch::new_tokensearch(
                 db,
                 ValueSearch::NotSome(val),
+                filters,
                 true,
                 true,
                 &query_fragment,
                 node_nr,
-                location_in_query,
             ),
             NodeSearchSpec::AnyToken => {
                 NodeSearch::new_anytoken_search(db, &query_fragment, node_nr)
@@ -359,11 +564,9 @@ impl<'a> NodeSearch<'a> {
 
                 Ok(NodeSearch {
                     it: Box::new(it),
-                    desc: Some(Desc::empty_with_fragment(
-                        super::NodeDescArg {
-                            query_fragment,
-                            node_nr,
-                        },
+                    desc: Some(ExecutionNodeDesc::empty_with_fragment(
+                        node_nr,
+                        query_fragment,
                         Some(est_output),
                     )),
                     node_search_desc: Arc::new(NodeSearchDesc {
@@ -384,6 +587,7 @@ impl<'a> NodeSearch<'a> {
         db: &'a AnnotationGraph,
         qname: (Option<String>, String),
         val: ValueSearch<String>,
+        filters: Vec<MatchValueFilterFunc>,
         is_meta: bool,
         query_fragment: &str,
         node_nr: usize,
@@ -426,8 +630,13 @@ impl<'a> NodeSearch<'a> {
 
         let est_output = match val {
             ValueSearch::Some(ref val) => {
-                db.get_node_annos()
-                    .guess_max_count(qname.0.as_deref(), &qname.1, &val, &val)
+                if qname.0.as_deref() == Some(ANNIS_NS) && qname.1 == NODE_NAME {
+                    // Our data model assumes that annis::node_name annotations are unique
+                    1
+                } else {
+                    db.get_node_annos()
+                        .guess_max_count(qname.0.as_deref(), &qname.1, val, val)
+                }
             }
             ValueSearch::NotSome(ref val) => {
                 let total = db
@@ -435,7 +644,7 @@ impl<'a> NodeSearch<'a> {
                     .number_of_annotations_by_name(qname.0.as_deref(), &qname.1);
                 total
                     - db.get_node_annos()
-                        .guess_max_count(qname.0.as_deref(), &qname.1, &val, &val)
+                        .guess_max_count(qname.0.as_deref(), &qname.1, val, val)
             }
             ValueSearch::Any => db
                 .get_node_annos()
@@ -447,36 +656,11 @@ impl<'a> NodeSearch<'a> {
 
         let it = base_it.map(|n| smallvec![n]);
 
-        let mut filters: Vec<MatchFilterFunc> = Vec::new();
-
-        match val {
-            ValueSearch::Any => {}
-            ValueSearch::Some(val) => {
-                filters.push(Box::new(move |m, node_annos| {
-                    if let Some(anno_val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
-                        anno_val == val.as_str()
-                    } else {
-                        false
-                    }
-                }));
-            }
-            ValueSearch::NotSome(val) => {
-                filters.push(Box::new(move |m, node_annos| {
-                    if let Some(anno_val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
-                        anno_val != val.as_str()
-                    } else {
-                        false
-                    }
-                }));
-            }
-        }
         Ok(NodeSearch {
             it: Box::new(it),
-            desc: Some(Desc::empty_with_fragment(
-                super::NodeDescArg {
-                    query_fragment: query_fragment.to_owned(),
-                    node_nr,
-                },
+            desc: Some(ExecutionNodeDesc::empty_with_fragment(
+                node_nr,
+                query_fragment.to_owned(),
                 Some(est_output),
             )),
             node_search_desc: Arc::new(NodeSearchDesc {
@@ -493,9 +677,9 @@ impl<'a> NodeSearch<'a> {
         qname: (Option<String>, String),
         pattern: &str,
         negated: bool,
+        filters: Vec<MatchValueFilterFunc>,
         is_meta: bool,
-        node_desc_arg: super::NodeDescArg,
-        location_in_query: Option<LineColumnRange>,
+        node_desc_arg: NodeDescArg,
     ) -> Result<NodeSearch<'a>> {
         // match_regex works only with values
         let base_it =
@@ -549,41 +733,13 @@ impl<'a> NodeSearch<'a> {
 
         let it = base_it.map(|n| smallvec![n]);
 
-        let mut filters: Vec<MatchFilterFunc> = Vec::new();
-
-        let full_match_pattern = graphannis_core::util::regex_full_match(&pattern);
-        let re = regex::Regex::new(&full_match_pattern);
-        match re {
-            Ok(re) => {
-                if negated {
-                    filters.push(Box::new(move |m, node_annos| {
-                        if let Some(val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
-                            !re.is_match(&val)
-                        } else {
-                            false
-                        }
-                    }));
-                } else {
-                    filters.push(Box::new(move |m, node_annos| {
-                        if let Some(val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
-                            re.is_match(&val)
-                        } else {
-                            false
-                        }
-                    }));
-                }
-            }
-            Err(e) => {
-                return Err(GraphAnnisError::AQLSemanticError(AQLError {
-                    desc: format!("/{}/ -> {}", pattern, e),
-                    location: location_in_query,
-                }));
-            }
-        }
-
         Ok(NodeSearch {
             it: Box::new(it),
-            desc: Some(Desc::empty_with_fragment(node_desc_arg, Some(est_output))),
+            desc: Some(ExecutionNodeDesc::empty_with_fragment(
+                node_desc_arg.node_nr,
+                node_desc_arg.query_fragment,
+                Some(est_output),
+            )),
             node_search_desc: Arc::new(NodeSearchDesc {
                 qname: (qname.0, Some(qname.1)),
                 cond: filters,
@@ -596,11 +752,11 @@ impl<'a> NodeSearch<'a> {
     fn new_tokensearch(
         db: &'a AnnotationGraph,
         val: ValueSearch<String>,
+        filters: Vec<MatchValueFilterFunc>,
         leafs_only: bool,
         match_regex: bool,
         query_fragment: &str,
         node_nr: usize,
-        location_in_query: Option<LineColumnRange>,
     ) -> Result<NodeSearch<'a>> {
         let it_base: Box<dyn Iterator<Item = Match>> = match val {
             ValueSearch::Any => {
@@ -623,7 +779,7 @@ impl<'a> NodeSearch<'a> {
                     db.get_node_annos().exact_anno_search(
                         Some(&TOKEN_KEY.ns),
                         &TOKEN_KEY.name,
-                        ValueSearch::Some(&val),
+                        ValueSearch::Some(val),
                     )
                 };
                 Box::new(it)
@@ -680,101 +836,6 @@ impl<'a> NodeSearch<'a> {
                 anno_key: NODE_TYPE_KEY.clone(),
             }]
         });
-        // create filter functions
-        let mut filters: Vec<MatchFilterFunc> = Vec::new();
-
-        match val {
-            ValueSearch::Some(ref val) => {
-                if match_regex {
-                    let full_match_pattern = graphannis_core::util::regex_full_match(val);
-                    let re = regex::Regex::new(&full_match_pattern);
-                    match re {
-                        Ok(re) => filters.push(Box::new(move |m, node_annos| {
-                            if let Some(val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
-                                re.is_match(&val)
-                            } else {
-                                false
-                            }
-                        })),
-                        Err(e) => {
-                            return Err(GraphAnnisError::AQLSemanticError(AQLError {
-                                desc: format!("/{}/ -> {}", val, e),
-                                location: location_in_query,
-                            }));
-                        }
-                    };
-                } else {
-                    let val = val.clone();
-                    filters.push(Box::new(move |m, node_annos| {
-                        if let Some(anno_val) = node_annos.get_value_for_item(&m.node, &m.anno_key)
-                        {
-                            anno_val == val.as_str()
-                        } else {
-                            false
-                        }
-                    }));
-                };
-            }
-            ValueSearch::NotSome(ref val) => {
-                if match_regex {
-                    let full_match_pattern = graphannis_core::util::regex_full_match(val);
-                    let re = regex::Regex::new(&full_match_pattern);
-                    match re {
-                        Ok(re) => filters.push(Box::new(move |m, node_annos| {
-                            if let Some(val) = node_annos.get_value_for_item(&m.node, &m.anno_key) {
-                                !re.is_match(&val)
-                            } else {
-                                false
-                            }
-                        })),
-                        Err(e) => {
-                            return Err(GraphAnnisError::AQLSemanticError(AQLError {
-                                desc: format!("/{}/ -> {}", val, e),
-                                location: location_in_query,
-                            }));
-                        }
-                    };
-                } else {
-                    let val = val.clone();
-                    filters.push(Box::new(move |m, node_annos| {
-                        if let Some(anno_val) = node_annos.get_value_for_item(&m.node, &m.anno_key)
-                        {
-                            anno_val != val.as_str()
-                        } else {
-                            false
-                        }
-                    }));
-                };
-            }
-            ValueSearch::Any => {}
-        };
-
-        if leafs_only {
-            let cov_gs: Vec<Arc<dyn GraphStorage>> = db
-                .get_all_components(Some(AnnotationComponentType::Coverage), None)
-                .into_iter()
-                .filter_map(|c| db.get_graphstorage(&c))
-                .filter(|gs| {
-                    if let Some(stats) = gs.get_statistics() {
-                        stats.nodes > 0
-                    } else {
-                        true
-                    }
-                })
-                .collect();
-
-            let filter_func: Box<
-                dyn Fn(&Match, &dyn AnnotationStorage<NodeID>) -> bool + Send + Sync,
-            > = Box::new(move |m, _| {
-                for cov in cov_gs.iter() {
-                    if cov.get_outgoing_edges(m.node).next().is_some() {
-                        return false;
-                    }
-                }
-                true
-            });
-            filters.push(filter_func);
-        };
 
         // TODO: is_leaf should be part of the estimation
         let est_output = match val {
@@ -823,11 +884,9 @@ impl<'a> NodeSearch<'a> {
 
         Ok(NodeSearch {
             it: Box::new(it),
-            desc: Some(Desc::empty_with_fragment(
-                super::NodeDescArg {
-                    query_fragment: query_fragment.to_owned(),
-                    node_nr,
-                },
+            desc: Some(ExecutionNodeDesc::empty_with_fragment(
+                node_nr,
+                query_fragment.to_owned(),
                 Some(est_output),
             )),
             node_search_desc: Arc::new(NodeSearchDesc {
@@ -849,7 +908,7 @@ impl<'a> NodeSearch<'a> {
     ) -> Result<NodeSearch<'a>> {
         let it: Box<dyn Iterator<Item = MatchGroup>> = Box::from(AnyTokenSearch::new(db)?);
         // create filter functions
-        let mut filters: Vec<MatchFilterFunc> = Vec::new();
+        let mut filters: Vec<MatchValueFilterFunc> = Vec::new();
 
         let cov_gs: Vec<Arc<dyn GraphStorage>> = db
             .get_all_components(Some(AnnotationComponentType::Coverage), None)
@@ -864,7 +923,7 @@ impl<'a> NodeSearch<'a> {
             })
             .collect();
 
-        let filter_func: MatchFilterFunc = Box::new(move |m, _| {
+        let filter_func: MatchValueFilterFunc = Box::new(move |m, _| {
             for cov in cov_gs.iter() {
                 if cov.get_outgoing_edges(m.node).next().is_some() {
                     return false;
@@ -882,11 +941,9 @@ impl<'a> NodeSearch<'a> {
 
         Ok(NodeSearch {
             it: Box::new(it),
-            desc: Some(Desc::empty_with_fragment(
-                super::NodeDescArg {
-                    query_fragment: query_fragment.to_owned(),
-                    node_nr,
-                },
+            desc: Some(ExecutionNodeDesc::empty_with_fragment(
+                node_nr,
+                query_fragment.to_owned(),
                 Some(est_output),
             )),
             node_search_desc: Arc::new(NodeSearchDesc {
@@ -904,7 +961,7 @@ impl<'a> NodeSearch<'a> {
     pub fn new_partofcomponentsearch(
         db: &'a AnnotationGraph,
         node_search_desc: Arc<NodeSearchDesc>,
-        desc: Option<&Desc>,
+        desc: Option<&ExecutionNodeDesc>,
         components: HashSet<Component<AnnotationComponentType>>,
         edge_anno_spec: Option<EdgeAnnoSearchSpec>,
     ) -> Result<NodeSearch<'a>> {
@@ -975,7 +1032,7 @@ impl<'a> NodeSearch<'a> {
         })
     }
 
-    pub fn set_desc(&mut self, desc: Option<Desc>) {
+    pub fn set_desc(&mut self, desc: Option<ExecutionNodeDesc>) {
         self.desc = desc;
     }
 
@@ -989,7 +1046,7 @@ impl<'a> ExecutionNode for NodeSearch<'a> {
         self
     }
 
-    fn get_desc(&self) -> Option<&Desc> {
+    fn get_desc(&self) -> Option<&ExecutionNodeDesc> {
         self.desc.as_ref()
     }
 
