@@ -1334,11 +1334,9 @@ where
 
                 // Get the covered text which either goes until the next token or until the end of the text if there is none
                 let mut covered_text_after = if let Some(end_pos) = whitespace_end_pos {
-                    if current_text_offset < end_pos {
-                        std::string::String::with_capacity(current_text_offset - token_right_char)
-                    } else {
-                        std::string::String::default()
-                    }
+                    std::string::String::with_capacity(
+                        end_pos.checked_sub(current_text_offset).unwrap_or(0),
+                    )
                 } else {
                     std::string::String::default()
                 };
